
#ifndef ROOT_TCanvas
#define ROOT_TCanvas

// Copyright (C) 1994 CodeCERN. All rights reserved.

//////////////////////////////////////////////////////////////////////////
//                                                                      //
// TCanvas                                                              //
//                                                                      //
// Graphics canvas.                                                     //
//                                                                      //
//////////////////////////////////////////////////////////////////////////

#ifndef ROOT_TPad
//*KEEP,TPad.
#include "TPad.h"
//*KEND.
#endif

#ifndef ROOT_TAttCanvas
//*KEEP,TAttCanvas.
#include "TAttCanvas.h"
//*KEND.
#endif

#ifndef ROOT_TGXW
//*KEEP,TGXW.
#include "TGXW.h"
//*KEND.
#endif

#ifndef ROOT_TString
//*KEEP,TString.
#include "TString.h"
//*KEND.
#endif

#ifndef ROOT_TCanvasImp
//*KEEP,TCanvasImp.
#include "TCanvasImp.h"
//*KEND.
#endif


enum ECanvasEditMode {
   kIsNotEditable,
   kIsEditable
};

class TContextMenu;
class TControlBar;

class TCanvas : public TPad {

friend class TCanvasImp;

protected:
   TString      fDISPLAY;         //Name of destination screen
   Int_t        fDoubleBuffer;    //Double buffer flag (0=off, 1=on)
   Bool_t       fRetained;        //Retain structure flag
   Size_t       fXsizeUser;       //User specified size of canvas along X in CM
   Size_t       fYsizeUser;       //User specified size of canvas along Y in CM
   Size_t       fXsizeReal;       //Current size of canvas along X in CM
   Size_t       fYsizeReal;       //Current size of canvas along Y in CM
   Int_t        fCanvasID;        //Canvas identifier
   Int_t        fCtopX;           //X top left corner of canvas in pixel coordinates
   Int_t        fCtopY;           //Y top left corner of canvas in pixel coordinates
   UInt_t       fCw;              //Width of the canvas along X (pixels)
   UInt_t       fCh;              //Height of the canvas along Y (pixels)
   TAttCanvas   fCatt;            //Canvas attributes
   TObject     *fSelected;        //Currently selected object
   TString      fSelectedOpt;     //Drawing option of selected object
   TVirtualPad *fSelectedPad;     //Pad containing currently selected object
   TVirtualPad *fPadSave;         //Pointer to saved pad in HandleInput
   Int_t        fEvent;           //Type of current or last handled event
   Bool_t       fShowEventStatus; //Show event status panel
   TControlBar *fEditorBar;       //Editor control bar
   TCanvasImp  *fCanvasImp;       //Window system specific canvas implementation
   Bool_t       fMoveOpaque;      //Move objects in opaque mode
   Bool_t       fResizeOpaque;    //Resize objects in opaque mode
   Color_t      fHighLightColor;  //Highlight color of active pad
   Bool_t       fBatch;           //True when in batchmode
   ECanvasEditMode fIsEditable;   //True if canvas is editable
   TContextMenu *fContextMenu;    //Context menu pointer

private:
   TCanvas(const TCanvas &canvas);  // cannot copy canvas, use TObject::Clone
   TCanvas &operator=(const TCanvas &rhs);  // idem
   void     Build();
   void     CopyPixmaps();
   void     DrawEventStatus(Int_t button, Int_t x, Int_t y, TObject *selected);

protected:
   virtual void ExecuteEvent(Int_t event, Int_t px, Int_t py);

public:
   TCanvas();
   TCanvas(Text_t *name, Text_t *title="", Int_t form=1);
   TCanvas(Text_t *name, Text_t *title, Int_t ww, Int_t wh);
   TCanvas(Text_t *name, Text_t *title,
           Int_t wtopx, Int_t wtopy, Int_t ww, Int_t wh);
   virtual ~TCanvas();

   void              cd(Int_t subpadnumber=0);
   void              Clear(Option_t *option="");
   void              Close(Option_t *option="");
   virtual void      Draw(Option_t *option="");
   virtual void      EditorBar();
   void              FeedbackMode(Bool_t set);
   void              Flush();
   void              UseCurrentStyle(); // *MENU*
   void              ForceUpdate() { fCanvasImp->ForceUpdate(); }
   const Text_t     *GetDISPLAY() const {return fDISPLAY.Data();}
   TContextMenu     *GetContextMenu(){return fContextMenu;};
   Int_t             GetDoubleBuffer() {return fDoubleBuffer;}
   TControlBar      *GetEditorBar() {return fEditorBar;}
   Int_t             GetEvent() const { return fEvent; }
   Color_t           GetHighLightColor() const { return fHighLightColor; }
   virtual void     *GetPadDivision(Int_t xdivision=1, Int_t ydivision=1);
   TVirtualPad      *GetPadSave() const { return fPadSave; }
   TObject          *GetSelected() {return fSelected;}
   const Option_t   *GetSelectedOpt() const {return fSelectedOpt.Data();}
   TVirtualPad      *GetSelectedPad() const { return fSelectedPad; }
   Bool_t            GetShowEventStatus() { return fShowEventStatus; }
   Size_t            GetXsizeUser() {return fXsizeUser;}
   Size_t            GetYsizeUser() {return fYsizeUser;}
   Size_t            GetXsizeReal() {return fXsizeReal;}
   Size_t            GetYsizeReal() {return fYsizeReal;}
   Int_t             GetCanvasID() {return fCanvasID;}
   TCanvasImp       *GetCanvasImp() {return fCanvasImp;}
   Int_t             GetWtopX() {return fCtopX;}
   Int_t             GetWtopY() {return fCtopY;}
   UInt_t            GetWw() {return fCw;}
   UInt_t            GetWh() {return fCh;}
   virtual void      GetCanvasPar(Int_t &wtopx, Int_t &wtopy, UInt_t &ww, UInt_t &wh)
                         {wtopx=fCtopX; wtopy=fCtopY; ww=fCw; wh=fCh;}
   virtual void      HandleInput(EEventType button, Int_t x, Int_t y);
   void              Iconify() { fCanvasImp->Iconify(); }
   Bool_t            IsBatch() { return fBatch; }
   ECanvasEditMode   IsEditable() { return fIsEditable; }
   Bool_t            IsRetained() {return fRetained;}
   virtual void      ls(Option_t *option="");
   void              MoveOpaque(Int_t set=1);
   Bool_t            OpaqueMoving() const { return fMoveOpaque; }
   Bool_t            OpaqueResizing() const { return fResizeOpaque; }
   virtual void      Paint(Option_t *option="");
   virtual void      Resize(Option_t *option="");
   void              ResizeOpaque(Int_t set=1) { fResizeOpaque = set; }
   void              SaveSource(Text_t *filename="", Option_t *option="");
   virtual void      SetCursor(ECursor cursor);
   virtual void      SetDoubleBuffer(Int_t mode=1);
   void              SetCanvas(UInt_t ww, UInt_t wh) { fCanvasImp->SetCanvas(ww, wh); }
   virtual void      SetEditable(ECanvasEditMode mode=kIsEditable) {fIsEditable = mode;} // *TOGGLE*
   void              SetHighLightColor(Color_t col) { fHighLightColor = col; }
   void              SetSelected(TObject *obj) {fSelected = obj;}
   void              Show() { fCanvasImp->Show(); }
   virtual void      Size(Float_t xsizeuser=0, Float_t ysizeuser=0);
   void              SetBatch(Bool_t batch=kTRUE);
   void              SetRetained(Bool_t retained=kTRUE) { fRetained=retained;}
   void              SetTitle(Text_t *title="");
   virtual void      ToggleEventStatus();
   virtual void      Update();

   static void       MakeDefCanvas();

   ClassDef(TCanvas,1)  //Graphics canvas
};

#endif

