
#ifndef ROOT_TPadView3D
#define ROOT_TPadView3D

// Copyright (C) 1997 CodeCERN. All rights reserved.

//////////////////////////////////////////////////////////////////////////
//                                                                      //
// TPadView3D                                                           //
//                                                                      //
// TPadView3D is a generic 3D viewer.                                   //
// For a concrete viewer see TGLViewer.                                 //
//                                                                      //
//////////////////////////////////////////////////////////////////////////

#ifndef ROOT_TObjectView
// +SEQ,TObjectView.
#endif
#ifndef ROOT_TObject
//*KEEP,TObject.
#include "TObject.h"
//*KEND.
#endif

class TVirtualPad;
class TPolyMarker3D;
class TPolyLine3D;
class TNode;


//class TPadView3D : public TObjectView
class TPadView3D
{
 protected:
   TVirtualPad  *fParent;            // Pointer to the original TPad object

   Double_t      fViewBoxMin[3];     // Minimum of clip box
   Double_t      fViewBoxMax[3];     // Maximum of clip box
   Double_t      fTranslate[3];      // The vector move object to center of scene
   Double_t      fExtraTranslate[3]; // The vector to move object with a mouse
   Double_t      fAngles[3];         // Latitude, Longitude, Psi
   Double_t      fExtraAngles[3];    // Latitude, Longitude, Psi
   Double_t      fAnglFactor[3];     // Latitude, Longitude, Psi
   Float_t       fScale;             // The scale factor to control the border of the clip box

 public:
   TPadView3D() { fParent = 0; }  //default ctor
   TPadView3D(TVirtualPad *pad) { SetPad(pad); }
   virtual ~TPadView3D();
   virtual void ExecuteEvent(Int_t event, Int_t px, Int_t py);
   TVirtualPad *GetPad() const { return fParent; }
   virtual void Paint(Option_t *option="");
   virtual void Size(Int_t width, Int_t height);
   virtual void PaintBeginModel(Option_t *opt="");
   virtual void PaintEnd(Option_t *opt="");
   virtual void PaintScene(Option_t *opt="");
   virtual void PaintPolyMarker(TPolyMarker3D *marker, Option_t *opt="");
   virtual void PaintPolyLine(TPolyLine3D *line, Option_t *opt="");
   virtual void PushMatrix() { }
   virtual void PopMatrix() { }
   virtual void SetAttNode(TNode *node, Option_t *opt="");
   void SetPad(TVirtualPad *pad=0) { fParent = pad; }
   virtual void UpdateNodeMatrix(TNode *node, Option_t *opt="");
   virtual void UpdateView() { }

//   ClassDef(TPadView3D,0)   //Generic 3D viewer
};

inline void TPadView3D::ExecuteEvent(Int_t, Int_t, Int_t) { }
inline void TPadView3D::Paint(Option_t *) { }
inline void TPadView3D::Size(Int_t, Int_t) { }
inline void TPadView3D::PaintBeginModel(Option_t *) { }
inline void TPadView3D::PaintEnd(Option_t *) { }
inline void TPadView3D::PaintScene(Option_t *) { }
inline void TPadView3D::PaintPolyMarker(TPolyMarker3D *, Option_t *) { }
inline void TPadView3D::PaintPolyLine(TPolyLine3D *, Option_t *) { }
inline void TPadView3D::SetAttNode(TNode *, Option_t *) { }
inline void TPadView3D::UpdateNodeMatrix(TNode *, Option_t *) { }

#endif
