
#ifndef ROOT_TPolyLine3D
#define ROOT_TPolyLine3D

// Copyright (C) 1994 CodeCERN. All rights reserved.

//////////////////////////////////////////////////////////////////////////
//                                                                      //
// TPolyLine3D                                                          //
//                                                                      //
// A 3-D PolyLine.                                                      //
//                                                                      //
//////////////////////////////////////////////////////////////////////////

#ifndef ROOT_TObject
//*KEEP,TObject.
#include "TObject.h"
//*KEND.
#endif
#ifndef ROOT_TTString
//*KEEP,TString.
#include "TString.h"
//*KEND.
#endif
#ifndef ROOT_TAttLine
//*KEEP,TAttLine.
#include "TAttLine.h"
//*KEND.
#endif
#ifndef ROOT_X3DBuffer
//*KEEP,X3DBuffer,T=C.
#include "X3DBuffer.h"
//*KEND.
#endif


class TPolyLine3D : public TObject, public TAttLine {

protected:
        Int_t        fN;            //Number of points
        Float_t      *fP;           //Array of 3-D coordinates  (x,y,z)
        TString      fOption;       //options
        UInt_t       fGLList;       // The list number for OpenGL view

public:
        TPolyLine3D();
        TPolyLine3D(Int_t n, Option_t *option="");
        TPolyLine3D(Int_t n, Float_t *p, Option_t *option="");
        TPolyLine3D(Int_t n, Float_t *x, Float_t *y, Float_t *z, Option_t *option="");
        TPolyLine3D(const TPolyLine3D &polylin);
        virtual ~TPolyLine3D();

        virtual void      Copy(TObject &polyline);
        virtual Int_t     DistancetoPrimitive(Int_t px, Int_t py);
        virtual void      Draw(Option_t *option="");
        virtual void      DrawPolyLine(Int_t n, Float_t *p, Option_t *option="");
        virtual void      ExecuteEvent(Int_t event, Int_t px, Int_t py);
        virtual void      GetPolyLine(Int_t &n, Float_t *p, Option_t *option);
        Int_t             GetN() {return fN;}
        Float_t           *GetP() {return fP;}
        const Option_t    *GetOption() const {return fOption.Data();}
        virtual Bool_t    Is3D() {return kTRUE;}
        virtual void      ls(Option_t *option="");
        virtual void      Paint(Option_t *option="");
        virtual void      PaintPolyLine(Int_t n, Float_t *p, Option_t *option="");
        virtual void      Print(Option_t *option="");
        virtual void      SavePrimitive(ofstream &out, Option_t *option);
        virtual void      SetOption(Option_t *option="") {fOption = option;}
        virtual void      SetPoint(Int_t point, Float_t x, Float_t y, Float_t z); // *MENU*
        virtual void      SetPolyLine(Int_t n, Float_t *p, Option_t *option="");
        virtual void      Sizeof3D() const;

        ClassDef(TPolyLine3D,1)  //A 3-D PolyLine
};

#endif

