
#ifndef ROOT_TProfile
#define ROOT_TProfile

// Copyright (C) 1995 CodeCERN. All rights reserved.

//////////////////////////////////////////////////////////////////////////
//                                                                      //
// TProfile                                                             //
//                                                                      //
// Profile histogram class.                                             //
//                                                                      //
//////////////////////////////////////////////////////////////////////////

#ifndef ROOT_TH1
//*KEEP,TH1.
#include "TH1.h"
//*KEND.
#endif

enum EErrorType { kERRORMEAN = 0, kERRORSPREAD, kERRORSPREADI, kERRORSPREADG };


class TProfile : public TH1D {

protected:
    TArrayD     fBinEntries;      //number of entries per bin
    EErrorType  fErrorMode;       //Option to compute errors
    Float_t     fYmin;            //Lower limit in Y (if set)
    Float_t     fYmax;            //Upper limit in Y (if set)

private:
   void Fill(Axis_t) { MayNotUse("Fill(Axis_t)"); }
   void Fill(Axis_t, Stat_t) { MayNotUse("Fill(Axis_t, Stat_t)"); }
   void Fill(Axis_t, Axis_t, Axis_t, Stat_t) { MayNotUse("Fill(Axis_t, Axis_t, Axis_t, Stat_t)"); }
   void FillN(Int_t, Axis_t *, Double_t *, Int_t) { MayNotUse("FillN(Int_t, Axis_t*, Double_t*, Int_t)"); }
   void SetBins(Int_t, Float_t, Float_t, Int_t, Float_t, Float_t)
      { MayNotUse("SetBins(Int_t, Float_t, Float_t, Int_t, Float_t, Float_t"); }

public:
    TProfile();
    TProfile(Text_t *name,Text_t *title,Int_t nbinsx,Axis_t xlow,Axis_t xup, Option_t *option="");
    TProfile(Text_t *name,Text_t *title,Int_t nbinsx,Axis_t xlow,Axis_t xup,Axis_t ylow,Axis_t yup,Option_t *option="");
    TProfile(Text_t *name,Text_t *title,Int_t nbinsx,Axis_t *xbins, Option_t *option="");
    TProfile(const TProfile &profile);
    virtual ~TProfile();
            void    BuildOptions(Float_t ymin, Float_t ymax, Option_t *option);
    virtual void    Copy(TObject &hnew);
    virtual void    DrawCopy(Option_t *option="");
    virtual void    Fill(Axis_t x, Axis_t y);
    virtual void    Fill(Axis_t x, Axis_t y, Stat_t w);
    virtual void    FillN(Int_t ntimes, Axis_t *x, Axis_t *y, Double_t *w, Int_t stride=1);
    virtual Stat_t  GetBinContent(Int_t bin);
    virtual Stat_t  GetBinError(Int_t bin);
     const Option_t *GetErrorOption() const;
    virtual void    Reset(Option_t *option="");
    virtual void    SetBins(Int_t nbins, Float_t xmin, Float_t xmax);
    virtual void    SetErrorOption(Option_t *option=""); // *MENU*

    ClassDef(TProfile,1)  //Profile histogram class
};

#endif

