
#ifndef ROOT_TServerSocket
#define ROOT_TServerSocket

// Copyright (C) 1996 CodeCERN. All rights reserved.

//////////////////////////////////////////////////////////////////////////
//                                                                      //
// TServerSocket                                                        //
//                                                                      //
// This class implements server sockets. A server socket waits for      //
// requests to come in over the network. It performs some operation     //
// based on that request and then possibly returns a result to the      //
// requester. The actual work is done via the TSystem class (either     //
// TUnixSystem, TWin32System or TMacSystem).                            //
//                                                                      //
//////////////////////////////////////////////////////////////////////////

#ifndef ROOT_TNamed
//*KEEP,TNamed.
#include "TNamed.h"
//*KEND.
#endif
#ifndef ROOT_TInetAddress
//*KEEP,TInetAddress,T=C++.
#include "TInetAddress.h"
//*KEND.
#endif
#ifndef ROOT_TSocket
//*KEEP,TSocket,T=C++.
#include "TSocket.h"
//*KEND.
#endif


class TServerSocket : public TNamed {

private:
   Int_t        fSocket;     // server socket descriptor
   TString      fService;    // name of service (matches port #)
   TInetAddress fAddress;    // local internet address and port #

   TServerSocket() { fSocket = -1; }
   TServerSocket(const TServerSocket &);
   void operator=(const TServerSocket &);
   const Option_t *GetOption() const { return TObject::GetOption(); }

public:
   enum { kDefaultBacklog = 10 };

   TServerSocket(Int_t port, Bool_t reuse = kFALSE, Int_t backlog = kDefaultBacklog);
   TServerSocket(const char *service, Bool_t reuse = kFALSE, Int_t backlog = kDefaultBacklog);
   virtual ~TServerSocket() { Close(); }

   virtual TSocket      *Accept();
   virtual void          Close(Option_t *opt="");
   virtual Int_t         GetDescriptor() const { return fSocket; }
   virtual TInetAddress  GetLocalInetAddress();
   virtual Int_t         GetLocalPort();
   virtual Bool_t        IsValid() const { return fSocket == -1 ? kFALSE : kTRUE; }
   virtual Int_t         SetOption(ESockOptions opt, Int_t val);
   virtual Int_t         GetOption(ESockOptions opt, Int_t &val);

   ClassDef(TServerSocket,1)  //This class implements server sockets
};

#endif
