
#ifndef ROOT_TStopwatch
#define ROOT_TStopwatch

// Copyright (C) 1995 CodeCERN. All rights reserved.

//////////////////////////////////////////////////////////////////////////
//                                                                      //
// TStopwatch                                                           //
//                                                                      //
// Stopwatch class. This class returns the real and cpu time between    //
// the start and stop events.                                           //
//                                                                      //
//////////////////////////////////////////////////////////////////////////

#ifndef ROOT_TObject
//*KEEP,TObject.
#include "TObject.h"
//*KEND.
#endif


class TStopwatch : public TObject {

private:
   enum EState { kUndefined, kStopped, kRunning };

   Double_t     fStartRealTime;   //wall clock start time
   Double_t     fStopRealTime;    //wall clock stop time
   Double_t     fStartCpuTime;    //cpu start time
   Double_t     fStopCpuTime;     //cpu stop time
   EState       fState;           //stopwatch state

public:
   TStopwatch();
   void        Start();
   void        Stop();
   void        Continue();
   Double_t    RealTime();
   Double_t    CpuTime();
   static Double_t GetRealTime();
   static Double_t GetCPUTime();
   void        Print(Option_t *option="");

   ClassDef(TStopwatch,0)  //A stopwatch which times real and cpu time
};

#endif
