
#ifndef ROOT_TTUBE
#define ROOT_TTUBE

// Copyright (C) 1995 CodeCERN. All rights reserved.

////////////////////////////////////////////////////////////////////////////
//                                                                        //
// TTUBE                                                                  //
//                                                                        //
// This tube has 3 parameters, the inside radius, the outside radius, and //
// the half length in z. Optional parameter is number of segments, also   //
// known as precision (default value is 20).                              //
//                                                                        //
////////////////////////////////////////////////////////////////////////////

#ifndef ROOT_TShape
//*KEEP,TShape.
#include "TShape.h"
//*KEND.
#endif



const Int_t kDivNum = 20;               //default number of divisions



class TTUBE : public TShape {

    protected:
        Float_t fRmin;        // inside radius
        Float_t fRmax;        // outside radius
        Float_t fDz;          // half length in z
        Int_t   fNdiv;        // number of segments (precision)

        Double_t   *fSiTab;     // Table of sin(fPhi1) .... sin(fPhil+fDphi1)
        Double_t   *fCoTab;     // Table of cos(fPhi1) .... cos(fPhil+fDphi1)

        virtual void    MakeTableOfCoSin();  // Create the table of the fSiTab; fCoTab
        virtual void    PaintGLPoints(Float_t *vertex);


    public:
        TTUBE();
        TTUBE(Text_t *name, Text_t *title, Text_t *material, Float_t rmin, Float_t rmax, Float_t dz);
        virtual ~TTUBE();

        virtual Int_t   DistancetoPrimitive(Int_t px, Int_t py);
        virtual Int_t   GetNumberOfDivisions () const {if (fNdiv) return fNdiv; else return kDivNum;}
        virtual void    Paint(Option_t *option);
        virtual void    SetNumberOfDivisions (Int_t ndiv);
        virtual void    SetPoints(Float_t *buff);
        virtual void    Sizeof3D() const;

        ClassDef(TTUBE,1)  //TUBE shape
};

#endif
